__author__ = 'alex'
from fabric.api import local
from base_builder import BaseBuilder
import os

class StylesheetsBuilder(BaseBuilder):

    def __init__(self, project_path):
        self.__inputs = []
        BaseBuilder.__init__(self, project_path)

    def set_output_file(self, path):
        self.__output_file = path

    def add_stylesheet(self, stylesheet):
        self.__inputs.append(stylesheet)

    def build(self):
        BaseBuilder.build(self)

        if self.__output_file is None:
            raise Exception('output_file required')
        if len(self.__inputs) == 0:
            raise Exception('No sources')

        builder_path = os.path.join(self.closure_base_path, 'google-closure-stylesheets', 'closure-stylesheets.jar')

        args = ''
        args += self.get_compiler_args_str()
        args += ' --output-file %s' % os.path.join(self.project_path, self.__output_file)
        args += ' '+','.join([os.path.join(self.project_path, src) for src in self.__inputs])

        local('java -jar %s %s' % (builder_path, args))


__author__ = 'alex'
from pipes import quote
import os

class BaseBuilder:

    def __init__(self, project_path):
        self.__compiler_args = []
        self.project_path = project_path
        self.__init_closure_path()

    def __init_closure_path(self):
        closure_paths_file = os.path.join(self.project_path, '.closure_paths')
        if not os.path.exists(closure_paths_file):
            raise Exception('Closure paths file not found. Please call bootstrap() before using builders')
        closure_paths = open(closure_paths_file, 'r').read()
        closure_paths = os.path.join(self.project_path, closure_paths)
        if not os.path.exists(closure_paths):
            raise Exception('Closure library not found. Please call bootstrap() before using builders')
        self.closure_base_path = closure_paths

    def set_compiler_path(self, path):
        self.__compiler_path = quote(path)

    def add_compiler_arg(self, name, value = None):
        self.__compiler_args.append(name)
        if value is not None:
            self.__compiler_args.append(value)

    def get_compiler_args_str(self):
        if len(self.__compiler_args) > 0:
            return ' '.join([quote(arg) for arg in self.__compiler_args])
        return ''

    def get_compiler_args(self):
        return self.__compiler_args

    def build(self):
        pass



__author__ = 'alex'

from bootstrap import bootstrap
from templates import TemplatesBuilder
from stylesheets import StylesheetsBuilder
from deps import DepsBuilder
from linter import Linter
from js import JSBuilder


__author__ = 'alex'

import os, urllib, zipfile
import platform
from fabric.api import local
from pipes import quote
from shutil import rmtree

"""
Module for installing closure library, templates and stylesheets into project.
TODO: windows support
"""

CLOSURE_LIBRARY_SVN = 'http://closure-library.googlecode.com/svn/trunk/'
CLOSURE_STYLESHEETS = 'http://closure-stylesheets.googlecode.com/files/closure-stylesheets-20111230.jar'
CLOSURE_TEMPLATES = 'http://closure-templates.googlecode.com/files/closure-templates-for-javascript-latest.zip'
CLOSURE_COMPILER = 'http://closure-compiler.googlecode.com/files/compiler-latest.zip'
PLOVR = 'http://plovr.googlecode.com/files/plovr-eba786b34df9.jar'

def __svn_up(path):
    try:
        local('svn up %s' % path)
    except StandardError:
        raise StandardError('svn not found. Please install console svn')

def __svn_checkout(url, path):
    try:
        local('svn checkout %s %s' % (url, quote(path)))
    except StandardError:
        raise StandardError('svn not found. Please install console svn')

def __download_and_unzip(url, path, dir_name):
    zip_path = os.path.join(path, dir_name + '.zip')

    if os.path.exists(zip_path):
        os.unlink(zip_path)

    urllib.urlretrieve(
        url,
        zip_path
    )

    target_path = os.path.join(path, dir_name)
    if os.path.exists(target_path):
        rmtree(target_path)

    zip = zipfile.ZipFile(zip_path)
    zip.extractall(path=target_path)
    zip.close()

    os.remove(zip_path)

def __append_to_gitignore(ignore_string, project_path):
    gitignore = os.path.join(project_path, '.gitignore')
    ignore_string += '\n'
    if os.path.exists(gitignore):
        content = open(gitignore, 'r').read()
        if not ignore_string in content:
            open(gitignore, "a").write(ignore_string)
    else:
        open(gitignore, "w").write(ignore_string)

def update_closure_library(path):
    lib_path = os.path.join(path, 'google-closure-library')
    if platform.system() == 'Windows':
        lib_path = '/'.join(lib_path.split('\\'))
    if os.path.exists(lib_path) and os.path.exists(os.path.join(lib_path,'.svn')):
        __svn_up(lib_path)
    else:
        __svn_checkout(CLOSURE_LIBRARY_SVN, lib_path)

def install_closure_stylesheets(path):
    print('Installing google closure stylesheets from %s' % CLOSURE_STYLESHEETS)
    stylesheets_path = os.path.join(path, 'google-closure-stylesheets')
    if os.path.exists(stylesheets_path):
        rmtree(stylesheets_path)
    os.mkdir(stylesheets_path)
    urllib.urlretrieve(
        CLOSURE_STYLESHEETS,
        os.path.join(stylesheets_path, 'closure-stylesheets.jar')
    )

def install_closure_compiler(path):
    print('Installing google closure compiler from %s' % CLOSURE_COMPILER)
    __download_and_unzip(CLOSURE_COMPILER, path, 'google-closure-compiler')

def install_plovr(path):
    print('Installing plovr from %s' % PLOVR)
    plovr_path = os.path.join(path, 'plovr')
    if os.path.exists(plovr_path):
        rmtree(plovr_path)
    os.mkdir(plovr_path)
    urllib.urlretrieve(
        PLOVR,
        os.path.join(plovr_path, 'plovr.jar')
    )

def install_closure_templates(path):
    print('Installing google closure templates from %s' % CLOSURE_TEMPLATES)
    __download_and_unzip(CLOSURE_TEMPLATES, path, 'google-closure-templates')

def bootstrap(project_path, dir_name=None, compiler=True, templates=True, stylesheets=True, library=True, plovr=True):
    """
    Setup google closure templates, stylesheets and library into project.
    """

    paths_file = '.closure_paths'
    if dir_name is None:
        if os.path.exists(paths_file):
            dir_name = open(paths_file, 'r').read()
        else:
            dir_name = 'google_closure'

    path = os.path.join(project_path, dir_name)
    if not os.path.exists(path):
        os.mkdir(path)

    if compiler:
        install_closure_compiler(path)
    if library:
        update_closure_library(path)
    if stylesheets:
        install_closure_stylesheets(path)
    if templates:
        install_closure_templates(path)
    if plovr:
        install_plovr(path)

    with open(paths_file, 'w') as paths:
        paths.write(dir_name)

    if os.path.exists(os.path.join(project_path, '.git')):
        __append_to_gitignore(ignore_string=dir_name + '/', project_path=project_path)
        __append_to_gitignore(ignore_string=paths_file, project_path=project_path)



from base_builder import BaseBuilder
import sys, os

class JSBuilder(BaseBuilder):

    def __init__(self, project_path, advanced=True):
        BaseBuilder.__init__(self, project_path)
        if advanced:
            self.add_compiler_arg('--compilation_level', 'ADVANCED_OPTIMIZATIONS')
            self.add_compiler_arg('--define', 'goog.DEBUG=false')

            self.add_compiler_arg('--jscomp_error', 'accessControls')
            self.add_compiler_arg('--jscomp_error', 'ambiguousFunctionDecl')
            self.add_compiler_arg('--jscomp_error', 'cast')
            self.add_compiler_arg('--jscomp_error', 'checkRegExp')
            self.add_compiler_arg('--jscomp_error', 'checkTypes')
            self.add_compiler_arg('--jscomp_error', 'checkVars')
            self.add_compiler_arg('--jscomp_error', 'const')
            self.add_compiler_arg('--jscomp_error', 'constantProperty')
            self.add_compiler_arg('--jscomp_error', 'deprecated')
            self.add_compiler_arg('--jscomp_error', 'duplicateMessage')
            self.add_compiler_arg('--jscomp_error', 'es5Strict')
            self.add_compiler_arg('--jscomp_error', 'externsValidation')
            self.add_compiler_arg('--jscomp_error', 'fileoverviewTags')
            self.add_compiler_arg('--jscomp_error', 'globalThis')
            self.add_compiler_arg('--jscomp_error', 'internetExplorerChecks')
            self.add_compiler_arg('--jscomp_error', 'invalidCasts')
            self.add_compiler_arg('--jscomp_error', 'misplacedTypeAnnotation')
            self.add_compiler_arg('--jscomp_error', 'missingProperties')
            self.add_compiler_arg('--jscomp_error', 'nonStandardJsDocs')
            self.add_compiler_arg('--jscomp_error', 'suspiciousCode')
            self.add_compiler_arg('--jscomp_error', 'strictModuleDepCheck')
            self.add_compiler_arg('--jscomp_error', 'typeInvalidation')
            self.add_compiler_arg('--jscomp_error', 'undefinedNames')
            self.add_compiler_arg('--jscomp_error', 'undefinedVars')
            self.add_compiler_arg('--jscomp_error', 'unknownDefines')
            self.add_compiler_arg('--jscomp_error', 'uselessCode')
            self.add_compiler_arg('--jscomp_error', 'visibility')

            self.add_compiler_arg('--warning_level', 'VERBOSE')
            self.add_compiler_arg('--summary_detail_level', '3')

            self.add_compiler_arg('--generate_exports')

    def set_output_file(self, path):
        self.__output_file = path

    def set_sources_folder(self, path):
        self.__sources_folder = path

    def set_main_file(self, file):
        self.__main_file = file

    def build(self):
        BaseBuilder.build(self)

        if self.__output_file is None:
            raise Exception('No output file specified')

        if self.__sources_folder is None:
            raise Exception('No sources folder specified')

        if self.__main_file is None:
            raise Exception('No main file specified')

        sys.path.append(os.path.join(self.closure_base_path, 'google-closure-library', 'closure', 'bin'))
        import calcdeps

        js_out = os.path.join(self.project_path, self.__output_file)
        js_src = os.path.join(self.project_path, self.__sources_folder)
        closure_src = os.path.join(self.closure_base_path, 'google-closure-library', 'closure', 'goog')
        closure_compiler = os.path.join(self.closure_base_path, 'google-closure-compiler', 'compiler.jar')

        args = list(self.get_compiler_args())

        args.append('--js_output_file')
        args.append(js_out)

        args.append('--js')
        args.append('%s/deps.js' % closure_src)

        search_paths = calcdeps.ExpandDirectories([js_src, closure_src])
        deps = calcdeps.CalculateDependencies(search_paths, [os.path.join(js_src, self.__main_file)])

        calcdeps.Compile(closure_compiler, deps, sys.stdout, args)
        print 'Built to %s' % js_out


from base_builder import BaseBuilder
from fabric.api import local
from pipes import quote
import os

class DepsBuilder(BaseBuilder):

    def set_output_file(self, path):
        self.__output_file = path

    def set_source(self, source):
        self.__source = source

    def set_custom_path_prefix(self, path):
        self.__custom_path_prefix = path

    def build(self):
        BaseBuilder.build(self)
        if self.__output_file is None:
            raise Exception('output file not specified')

        deps_writer = os.path.join(self.closure_base_path, 'google-closure-library', 'closure', 'bin', 'build', 'depswriter.py')

        args = ''
        args += self.get_compiler_args_str()
        args += ' --output_file=%s' % os.path.join(self.project_path, self.__output_file)
        if not(self.__source is None):
            folder = os.path.join(self.project_path, self.__source)
            args += " --root_with_prefix='%s %s'" % (quote(folder), quote(self.__get_path_prefix()))

        local('python %s %s' % (deps_writer, args))

    def __get_path_prefix(self):
        if self.__custom_path_prefix is None:
            base_js_path = os.path.join(self.closure_base_path, 'google-closure-library', 'closure', 'goog')
            folder = os.path.join(self.project_path, self.__source)
            return os.path.relpath(folder, base_js_path)
        else:
            return self.__custom_path_prefix



__author__ = 'alex'
from fabric.api import local
from base_builder import BaseBuilder
import os

class TemplatesBuilder(BaseBuilder):

    def __init__(self, project_path, use_goog=False):
        self.__inputs = []
        self.__deps = []

        BaseBuilder.__init__(self, project_path)
        if use_goog:
            self.add_compiler_arg('--shouldProvideRequireSoyNamespaces')
            self.add_compiler_arg('--shouldGenerateJsdoc')

    def set_output_path_format(self, path):
        self.__output_path_format = path

    def add_template(self, path):
        self.__inputs.append(path)

    def add_dep(self, path):
        self.__deps.append(path)

    def build(self):
        BaseBuilder.build(self)
        if self.__output_path_format is None:
            raise Exception('output_path_format required')
        if len(self.__inputs) == 0:
            raise Exception('No sources')

        builder_path = os.path.join(self.closure_base_path, 'google-closure-templates', 'SoyToJsSrcCompiler.jar')

        args = ''
        args += self.get_compiler_args_str()
        args += ' --outputPathFormat %s' % os.path.join(self.project_path, self.__output_path_format)
        args += ' --srcs %s' % ','.join([os.path.join(self.project_path, src) for src in self.__inputs])
        if len(self.__deps) > 0:
            args += ' --deps %s' % ','.join([os.path.join(self.project_path, src) for src in self.__deps])
        local('java -jar %s %s' % (builder_path, args))